@@ -19,8 +19,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-from __future__ import print_function
-
+import glob
+import grp
 import gzip
 import io
 import itertools
@@ -252,17 +252,6 @@ class Install(install_misc.InstallBase):
         if 'UBIQUITY_OEM_USER_CONFIG' not in os.environ:
             self.install_restricted_extras()
 
-        self.db.progress('INFO', 'ubiquity/install/apt_clone_restore')
-        try:
-            self.apt_clone_restore()
-        except Exception:
-            syslog.syslog(
-                syslog.LOG_WARNING,
-                'Could not restore packages from the previous install:')
-            for line in traceback.format_exc().split('\n'):
-                syslog.syslog(syslog.LOG_WARNING, line)
-            self.db.input('critical', 'ubiquity/install/broken_apt_clone')
-            self.db.go()
         try:
             self.copy_network_config()
         except Exception:
@@ -796,15 +785,6 @@ class Install(install_misc.InstallBase):
             script += '-oem'
         misc.execute(script)
 
-        osextras.unlink_force(self.target_file('etc/popularity-contest.conf'))
-        try:
-            participate = self.db.get('popularity-contest/participate')
-            install_misc.set_debconf(
-                self.target, 'popularity-contest/participate', participate,
-                self.db)
-        except debconf.DebconfError:
-            pass
-
         osextras.unlink_force(self.target_file('etc/papersize'))
         subprocess.call(['log-output', '-t', 'ubiquity', 'chroot', self.target,
                          'ucf', '--purge', '/etc/papersize'],
@@ -836,7 +816,6 @@ class Install(install_misc.InstallBase):
             pass
 
         packages = ['linux-image-' + self.kernel_version,
-                    'popularity-contest',
                     'libpaper1',
                     'ssl-cert']
         arch, subarch = install_misc.archdetect()
@@ -1009,6 +988,13 @@ class Install(install_misc.InstallBase):
                                 self.db.set('grub-installer/bootdev', response)
                         else:
                             break
+                    if arch == 'amd64' and subarch != 'efi' and os.path.ismount("/target/boot/efi"):
+                        dbfilter = grubinstaller.GrubInstaller(
+                            None, self.db, extra_args=['amd64/efi'])
+                        ret = dbfilter.run_command(auto_process=True)
+                        if ret != 0:
+                            raise install_misc.InstallStepError(
+                                "GrubInstaller failed with code %d" % ret)
                 else:
                     raise install_misc.InstallStepError(
                         "No bootloader installer found")
@@ -1309,7 +1295,7 @@ class Install(install_misc.InstallBase):
         # enabled
         cache = Cache()
         filtered_extra_packages = install_misc.query_recorded_installed()
-        for package in filtered_extra_packages.copy():
+        for package in sorted(filtered_extra_packages):
             pkg = cache.get(package)
             if not pkg:
                 continue
@@ -1321,7 +1307,8 @@ class Install(install_misc.InstallBase):
                     filtered_extra_packages.remove(package)
                     break
 
-        self.do_install(filtered_extra_packages)
+        # An ordered list from the set() to avoid the random dependencies failure.
+        self.do_install(sorted(filtered_extra_packages))
 
         if self.db.get('ubiquity/install_oem') == 'true':
             try:
@@ -1329,7 +1316,7 @@ class Install(install_misc.InstallBase):
                 # upgrade them to their versions in the OEM archive.
                 with open('/run/ubuntu-drivers-oem.autoinstall', 'r') as f:
                     oem_pkgs = set(f.read().splitlines())
-                    for oem_pkg in oem_pkgs.copy():
+                    for oem_pkg in sorted(oem_pkgs):
                         target_sources_list = self.target_file("etc/apt/sources.list.d/{}.list".format(oem_pkg))
                         if not os.path.exists(target_sources_list):
                             continue
@@ -1341,7 +1328,8 @@ class Install(install_misc.InstallBase):
                             syslog.syslog("Failed to apt update {}".format(target_sources_list))
                             oem_pkgs.discard(oem_pkg)
                     if oem_pkgs:
-                        self.do_install(oem_pkgs)
+                        # An ordered list from the set() to avoid the random dependencies failure.
+                        self.do_install(sorted(oem_pkgs))
             except FileNotFoundError:
                 pass
 
@@ -1634,31 +1622,6 @@ class Install(install_misc.InstallBase):
                 for line in installed:
                     print(line, file=fp)
 
-    def apt_clone_restore(self):
-        if 'UBIQUITY_OEM_USER_CONFIG' in os.environ:
-            return
-        import lsb_release
-        working = self.target_file('ubiquity-apt-clone')
-        working = os.path.join(working,
-                               'apt-clone-state-%s.tar.gz' % os.uname()[1])
-        codename = lsb_release.get_distro_information()['CODENAME']
-        if not os.path.exists(working):
-            return
-        install_misc.chroot_setup(self.target)
-        binds = ("/proc", "/sys", "/dev", "/run")
-        try:
-            for bind in binds:
-                misc.execute('mount', '--bind', bind, self.target + bind)
-            restore_cmd = [
-                'apt-clone', 'restore-new-distro',
-                working, codename, '--destination', self.target]
-            subprocess.check_call(
-                restore_cmd, preexec_fn=install_misc.debconf_disconnect)
-        finally:
-            install_misc.chroot_cleanup(self.target)
-            for bind in binds:
-                misc.execute('umount', '-f', self.target + bind)
-
     def copy_network_config(self):
         if 'UBIQUITY_OEM_USER_CONFIG' in os.environ:
             return
@@ -1687,6 +1650,18 @@ class Install(install_misc.InstallBase):
 
                 shutil.copy(source_network, target_network)
 
+        source_netplan = "/etc/netplan"
+        target_netplan = "/target" + source_netplan
+        if os.path.exists(source_netplan) and os.path.exists(target_netplan):
+            for cfg in glob.glob("90-NM-*", root_dir=source_netplan):
+                source_cfg = os.path.join(source_netplan, cfg)
+                target_cfg = os.path.join(target_netplan, cfg)
+
+                if os.path.exists(target_cfg):
+                    continue
+
+                shutil.copy(source_cfg, target_cfg)
+
     def copy_bluetooth_config(self):
         if 'UBIQUITY_OEM_USER_CONFIG' in os.environ:
             return
@@ -1788,14 +1763,15 @@ class Install(install_misc.InstallBase):
 
         for log_file in ('/var/log/syslog', '/var/log/partman',
                          '/var/log/installer/version', '/var/log/casper.log',
-                         '/var/log/installer/debug'):
+                         '/var/log/installer/debug', '/run/casper-md5check.json'):
             target_log_file = os.path.join(target_dir,
                                            os.path.basename(log_file))
             if os.path.isfile(log_file):
                 if not misc.execute('cp', '-a', log_file, target_log_file):
                     syslog.syslog(syslog.LOG_ERR,
                                   'Failed to copy installation log file')
-                os.chmod(target_log_file, stat.S_IRUSR | stat.S_IWUSR)
+                os.chmod(target_log_file, 0o640)
+                os.chown(target_log_file, -1, grp.getgrnam('adm').gr_gid)
         media_info = '/cdrom/.disk/info'
         if os.path.isfile(media_info):
             try:
@@ -1829,12 +1805,6 @@ class Install(install_misc.InstallBase):
                     print(oem_id, file=oem_id_file)
         except (debconf.DebconfError, IOError):
             pass
-        try:
-            path = self.target_file('ubiquity-apt-clone')
-            if os.path.exists(path):
-                shutil.move(path, self.target_file('var/log/installer'))
-        except IOError:
-            pass
 
     def save_random_seed(self):
         """Save random seed to the target system.
